#ifndef	_Array_h
#	define	_Array_h

extern	"C"
	{
#	include	<stdio.h>
#	include	<stdlib.h>
#	include	<unistd.h>
#	include	<memory.h>
	}
template	<class	T>
class	ArrayAccess
	{
	public:
		virtual	T	&	ElementAt(int	Index) = 0;
		virtual	T	&	operator	[]	(int	Index) = 0;
		virtual	bool	Append(T	&Value) = 0;
		virtual	T	&	Add(T	&Value) = 0;
		virtual	T	&	Add(const	T	&Value) = 0;
		virtual	bool	RemoveAt(int	Index) = 0;
		virtual	bool	InsertAt(T	&t, int	Index) = 0;
		virtual	bool	Clear() = 0;
		virtual	int		GetSize() = 0;
		virtual	bool	WriteToDisk(FILE*) {return(false);}
		virtual	bool	ReadFromDisk(FILE*) {return(false);}
	};

template	<class	T>
class	Array//	:
	//public	ArrayAccess<T>
	{
	private:
		T		*Storage;
		int		MaxSize;
		int		CurrentSize;
		int		Base;

	public:
		// Support object persistence
		bool	WriteToDisk(
			FILE	*fp)
			{
			if(Storage)
				{
				fwrite(&CurrentSize, 1, sizeof(int), fp);
				fwrite(Storage, sizeof(T), CurrentSize, fp);
				return(true);
				}
			return(false);
			}
		bool	ReadFromDisk(
			FILE	*fp)
			{
			int	theSize;
			fread(&theSize, 1, sizeof(int), fp);
			Resize(theSize);
			fread(Storage, sizeof(T), theSize, fp);
			CurrentSize = theSize;
			return(true);		
			}
		Array (int	tMaxSize=10)
			{
			Base = 0;
			MaxSize = 0;
			Storage = NULL;
			CurrentSize = 0;
			Resize(tMaxSize);
			}
		bool	Resize(int	tMaxSize)
			{
#ifdef	ENABLE_LIBRARY_CHECKS
			if ( tMaxSize > 1000000 )
				{
				printf("tMaxSize > 1000000 are you sure you want to this to happen?\n");
				assert(0);
				}
#endif
			if(tMaxSize!=0)
				{
				T	*tStorage = new T[tMaxSize];
				if(Storage)
					{
					if(tMaxSize<MaxSize)
						{
						memcpy(tStorage, Storage, sizeof(T)*tMaxSize);
						}
					else
						{
						memcpy(tStorage, Storage, sizeof(T)*MaxSize);
						}
					delete Storage;
					}
				MaxSize = tMaxSize;
				Storage = tStorage;
				}
			else
				{
				if(Storage)
					delete Storage;
				Storage = NULL;
				MaxSize = 0;
				CurrentSize = 0;
				}
			return(true);
			}
		~Array()
			{
			Resize(0);
			}
		inline	T	&	ElementAt(int	Index)
			{
#ifdef	ENABLE_LIBRARY_CHECKS
			if(Index>=CurrentSize)
				FatalError("Invalid get");
#endif
			return(Storage[Index + Base]);
			}
		inline	T	&	Get(int	Index)
			{
#ifdef	ENABLE_LIBRARY_CHECKS
			if(Index>=CurrentSize)
				FatalError("Invalid get");
#endif
			return(Storage[Index + Base]);
			}
		inline	T	&	ElementAti(int	Index)
			{
			if((Index+Base)>=MaxSize)
				{
				if((MaxSize*2) > (Index+Base) )
					Resize(MaxSize*2);
				else
					Resize(Index+Base+10);
				}
			if(Index>=CurrentSize)
				{
				CurrentSize=(Index+1);
				}
			return(Storage[Index+Base]);
			}
		inline	T	&	operator	[]	(int	Index)
			{
			return(ElementAt(Index));
			}
		bool	Append(T	&Value)
			{
			ElementAti(CurrentSize) = Value;
			return(true);
			}
		T	&	Add(T	&Value)
			{
			Append(Value);
			return(ElementAt(CurrentSize-1));
			}	
		T	&	Add(const	T	&Value)
			{
			T	v = Value;
			Append(v);
			return(ElementAt(CurrentSize-1));
			}
		bool	RemoveAt(int	Index)
			{
			if(Index>=CurrentSize)
				return(true);
			// Removal, is painful

			if ( Index == 0 )
				{
				++Base;
				if ( Base > 64 )
					{

					memcpy(&Storage[0], &Storage[Base], sizeof(T)*(CurrentSize-1));

					//for(int	i=0;i<(CurrentSize-1);i++) Storage[i] = Storage[i+Base];

					Base = 0;
					}
				--CurrentSize;
				return(true);
				}	

			if ( (Index - 1) != CurrentSize)
				memcpy(&Storage[Index+Base], &Storage[Index+Base+1],
					sizeof(T)*(CurrentSize-Index-1));

			--CurrentSize;	
			return(true);
			}
		bool	InsertAt(T	&t, int	Index)
			{
			if(Index>=CurrentSize)
				{
				ElementAti(Index) = t;
				return(true);
				}
			// deletion is even more painful
			// go from the end down to Index moving everyone up
			int		End = CurrentSize;
			while(End>Index)
				{
				ElementAti(End) = ElementAti(End-1);
				--End;
				}
			ElementAt(Index) = t;
			return(true);
			}
		bool	Clear()
			{
			CurrentSize=0;
			Base = 0;
			return(true);
			}
		inline	int		GetSize()
			{
			return(CurrentSize);
			}	
		int		CountMatches(
			T		&t)
			{
			int		Matches = 0;
			int		Index = 0;
			while(Index < GetSize())
				{
				if(ElementAt(Index)==t)
					++Matches;
				++Index;
				}
			return(Matches);
			}
	};

#endif


