#ifndef	_Fifo_h
#	define	_Fifo_h

#	include	"Array.h"

template	<class	T>
class	Fifo
	{
	public:
		Array<T>	theFifo;
		int			Head, Tail, Size;
		int			Valid;

		Fifo(int	aSize = 16)
			{
			int		Index;
			Size = aSize;

			Index = 0;
			while ( Index < Size )
				{
				theFifo.Add((T)0);
				++Index;
				}
			Head = Tail = Valid = 0;
			}
		bool	Resize(int	aSize)
			{
			theFifo.Clear();
			int		Index;
			Size = aSize;

			Index = 0;
			while ( Index < Size )
				{
				theFifo.Add((T)0);
				++Index;
				}
			Head = Tail = Valid = 0;
			return(true);
			}
		inline	bool	Clear()
			{
			Head = Tail = Valid = 0;
			return ( true );
			}
		inline	bool	Insert (T	t)
			{
#ifdef	ENABLE_LIBRARY_CHECKS
			if ( Valid == Size )
				{
				FatalError("Insert when full");
				return(false);
				}
#endif
			theFifo[Head] = t;
			Head = (Head+1);
			if ( Head >= Size )
				Head = 0;
			++Valid;
			return(true);
			}
		inline	bool	PushBack(T	t)
			{
#ifdef	ENABLE_LIBRARY_CHECKS
			if ( Valid == Size )
				{
				FatalError("PushBack when full");
				return(false);
				}
#endif
			if ( Tail == 0 )
				Tail = Size - 1;
			else
				--Tail;
			theFifo[Tail] = t;
			++Valid;
			return ( true );		
			}
		inline	bool	Push(T	t)
			{
			return(Insert(t));
			}
		inline	bool	Remove(T	*t)
			{
#ifdef	ENABLE_LIBRARY_CHECKS
			if ( ! Valid )
				{
				(*t) = 0;
				FatalError("Remove when empty");
				return(false);
				}
#endif
			(*t) = theFifo[Tail];
			Tail = (Tail+1)%Size;
			--Valid;
			return(true);
			}
		T		&		Pop()
			{
			int	oldTail = Tail;
			Tail = (Tail+1);
			if ( Tail >= Size )
				Tail = 0;
			--Valid;
			return(theFifo[oldTail]);
			}
		inline	bool	Peek(T		*t)
			{
#ifdef	ENABLE_LIBRARY_CHECKS
			if ( ! Valid )
				{
				(*t) = 0;
				FatalError("peek when empty");
				return ( false );
				}
#endif
			(*t) = theFifo[Tail];
			return(true);
			}
		inline	bool	PeekAt(T	*t, int	Index = 0)
			{
#ifdef	ENABLE_LIBRARY_CHECKS
			if ( Index >= CurrentSize() )
				{
				(*t) = 0;
				FatalError("peekat when empty");
				return ( false );
				}
#endif
			(*t) = theFifo[ (Tail + Index)%Size ];
			return(true);
			}
		inline	int		MaxSize ()
			{
			return(Size);
			}
		inline	int		CurrentSize()
			{
			return ( Valid );
			}
		inline	int		GetSize()
			{
			return ( Valid );
			}
		inline	int		FreeSpace()
			{
			return ( MaxSize() - CurrentSize() );
			}
		
	};

#endif
